<?php

namespace Drupal\site_settings\Tests;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Tests\BrowserTestBase;

/**
 * Tests the loading of Site Settings.
 *
 * @group SiteSettings
 */
class SiteSettingsUiTest extends BrowserTestBase {
  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Module list.
   *
   * @var array
   */
  protected static $modules = [
    'site_settings',
    'site_settings_sample_data',
    'field_ui',
    'user',
    'language',
  ];

  /**
   * Admin user.
   *
   * @var \Drupal\user\Entity\User|false
   */
  private $adminUser;

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function setUp():void {
    parent::setUp();

    // Create the user and login.
    $this->adminUser = $this->createUser([], NULL, TRUE);
    $this->drupalLogin($this->adminUser);
  }

  /**
   * Test site settings admin visibility.
   */
  public function testSiteSettingsAdminVisibility() {
    // Open the site settings list page.
    $this->drupalGet('admin/content/site-settings');
    $session = $this->assertSession();

    // Make sure the groups match.
    $session->responseContains('Images');
    $session->responseContains('Other');

    // Make sure the test plain text is as expected.
    $session->pageTextContains('Test plain text');

    // Make sure the test textarea is as expected.
    $session->pageTextContains('Test textarea name');

    // Make sure the test multiple entries contents are as expected.
    $session->pageTextContains('Test multiple entries');
    $session->pageTextContains('Test multiple entries name 2');

    // Make sure the test multiple entries and fields contents are as expected.
    $session->pageTextContains('Test multiple entries and fields name 1');
    $session->pageTextContains('Test multiple entries and fields name 2');

    // Make sure the test multiple fields contents are as expected.
    $session->pageTextContains('Test multiple fields name');

    // Make sure the test image is as expected.
    $session->pageTextContains('Test image');
    $session->pageTextContains('Test images 1');
    $session->pageTextContains('Test file');

  }

  /**
   * Test site settings add another.
   */
  public function testSiteSettingsAddAnother() {
    // Open the site settings list page.
    $this->drupalGet('admin/content/site-settings');

    // Click add another link.
    $this->click('#site-setting-3 a:contains("Add another")');
    $session = $this->assertSession();

    // Make sure we can see the expected form.
    $session->pageTextContains('Test multiple entries');
    $session->pageTextContains('Testing');
    $params = [
      'field_testing[0][value]' => 'testSiteSettingsAddAnother',
    ];
    $this->submitForm($params, 'Save');
    $session = $this->assertSession();

    // Ensure we saved correctly.
    $session->pageTextContains('Created the Test multiple entries Site Setting.');
    $session->pageTextContains('testSiteSettingsAddAnother');
  }

  /**
   * Test site settings edit existing.
   */
  public function testSiteSettingsEditExisting() {
    // Open the site settings list page.
    $this->drupalGet('admin/content/site-settings');
    $session = $this->assertSession();

    // Click add another link.
    $this->click('#site-setting-1 a:contains("Edit")');
    $session = $this->assertSession();

    // Make sure we can see the expected form.
    $session->pageTextContains('Test plain text');
    $session->pageTextContains('Testing');
    $params = [
      'field_testing[0][value]' => 'testSiteSettingsEditExisting',
    ];
    $this->submitForm($params, 'Save');
    $session = $this->assertSession();

    // Ensure we saved correctly.
    $session->pageTextContains('Saved the Test plain text Site Setting.');
    $session->pageTextContains('testSiteSettingsEditExisting');
  }

  /**
   * Test site settings create new type and add a setting to that.
   */
  public function testSiteSettingsCreateNewTypeAndSetting() {
    // Open the site settings list page.
    $this->drupalGet('admin/structure/site_setting_entity_type/add');

    // Create the new site setting.
    $params = [
      'label' => 'testSiteSettingsCreateNewTypeAndSetting',
      'id' => 'testsitesettingscreatenew',
      'existing_group' => 'other',
    ];
    $this->submitForm($params, 'Save');
    $session = $this->assertSession();

    // Ensure we saved correctly.
    $session->pageTextContains('Created the testSiteSettingsCreateNewTypeAndSetting Site Setting type.');

    // Add field.
    $this->drupalGet('admin/structure/site_setting_entity_type/testsitesettingscreatenew/edit/fields/add-field');
    $params = [
      'new_storage_type' => 'plain_text',
      'label' => 'testSiteSettingsCreateNewTypeAndSettingLabel',
      'field_name' => 'testing_again',
    ];
    $this->submitForm($params, 'Change field group');

    $params['group_field_options_wrapper'] = 'string';
    $this->submitForm($params, 'Continue');

    // Save field settings.
    $params = [];
    $this->submitForm($params, 'Save settings');
    $session = $this->assertSession();

    // Ensure we saved correctly.
    $session->pageTextContains('Saved testSiteSettingsCreateNewTypeAndSettingLabel configuration.');

    // Open the site settings list page.
    $this->drupalGet('admin/content/site-settings');
    $session = $this->assertSession();

    // Click add another link.
    $this->clickLink('Create setting');
    $session->pageTextContains('testSiteSettingsCreateNewTypeAndSettingLabel');
    $params = [
      'field_testing_again[0][value]' => 'testSiteSettingsCreateNewTypeAndSettingValue',
    ];
    $this->submitForm($params, 'Save');
    $session = $this->assertSession();

    // Ensure we saved correctly.
    $session->pageTextContains('Created the testSiteSettingsCreateNewTypeAndSetting Site Setting.');
    $session->pageTextContains('testSiteSettingsCreateNewTypeAndSettingValue');
  }

  /**
   * Test site settings groups.
   */
  public function testSiteSettingsUpdateGroup() {
    // Open the site settings list page.
    $this->drupalGet('admin/content/site-settings');

    // Click on the group type 'Boolean'.
    $this->clickLink('Boolean');

    // Change the group label.
    $params = [
      'label' => 'testSiteSettingsUpdateGroupLabel',
    ];
    $this->submitForm($params, 'Save site settings group entity type');
    $session = $this->assertSession();
    $session->pageTextContains('The site settings group entity type testSiteSettingsUpdateGroupLabel has been updated');

    // Open the site settings list page.
    $this->drupalGet('admin/content/site-settings');

    // Ensure that both groups were updated.
    $session = $this->assertSession();
    $session->elementTextContains('css', '#site-setting-11', 'testSiteSettingsUpdateGroupLabel');
    $session->elementTextContains('css', '#site-setting-12', 'testSiteSettingsUpdateGroupLabel');
  }

}
