<?php

namespace Drupal\site_settings\Twig;

use Drupal\Component\Uuid\Uuid;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\site_settings\SiteSettingsLoaderPluginManager;
use Twig\Extension\AbstractExtension;
use Twig\TwigFunction;

/**
 * Twig extension to load site settings.
 *
 * This is primarily aimed at the full site settings loader,
 * but can be used by any loader.
 */
class TwigExtension extends AbstractExtension {

  /**
   * Constructs the Site Settings Twig Extension.
   */
  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
    protected SiteSettingsLoaderPluginManager $pluginManager
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public function getFunctions(): array {
    return [
      new TwigFunction('site_setting', [$this, 'siteSetting']),
      new TwigFunction('all_site_settings', [$this, 'allSiteSettings']),
      new TwigFunction('site_settings_by_group', [$this, 'siteSettingsByGroup']),
      new TwigFunction('site_settings_by_name', [$this, 'siteSettingsByName']),
    ];
  }

  /**
   * Returns the render array of a site setting entity.
   *
   * Duplicates twig_tweak drupal_entity(), avoiding having
   * twig_tweak as a dependency. This loads the full entity
   * regardless of the chosen site settings loader.
   *
   * @see \Drupal\twig_tweak\TwigExtension::drupalEntity()
   */
  public function siteSetting(
    string $selector,
    string $view_mode = 'full',
    ?string $langcode = NULL,
    bool $check_access = TRUE
  ): array {
    $storage = $this->entityTypeManager->getStorage('site_setting_entity');

    // Load by UUID, falling back to ID.
    if (Uuid::isValid($selector)) {
      $entities = $storage->loadByProperties(['uuid' => $selector]);
      $entity = reset($entities);
    }
    else {
      $entity = $storage->load($selector);
    }

    if ($entity) {
      $build = [];

      // Check access on the entity.
      if ($check_access) {
        $access = $entity->access('view', NULL, TRUE);
      }
      else {
        $access = AccessResult::allowed();
      }
      if ($access->isAllowed()) {

        // Render the entity in the specified view mode and
        // language code.
        $build = $this->entityTypeManager
          ->getViewBuilder($entity->getEntityTypeId())
          ->view($entity, $view_mode, $langcode);
      }

      // Add the access and single site setting to the
      // cache tags.
      CacheableMetadata::createFromRenderArray($build)
        ->addCacheableDependency($access)
        ->addCacheableDependency($entity)
        ->applyTo($build);
      return $build;
    }

    return [];
  }

  /**
   * Returns a render array of a site setting's entities by machine name.
   *
   * @param string $selector
   *   The uuid or id of the entity to load.
   * @param string $view_mode
   *   The view mode to render.
   * @param string $langcode
   *   The langcode to render the view mode in.
   *
   * @return array
   *   The render array or an empty array.
   */
  public function siteSettingsByName(
    string $selector,
    string $view_mode = 'full',
    ?string $langcode = NULL,
  ): array {
    $storage = $this->entityTypeManager->getStorage('site_setting_entity');

    // Load by machine name.
    $site_settings = $storage->loadByProperties(['type' => $selector]);

    if (!empty($site_settings) && reset($site_settings) instanceof EntityInterface) {

      // Render the entities in the specified view mode and
      // language code.
      $build = $this->entityTypeManager
        ->getViewBuilder('site_setting_entity')
        ->viewMultiple($site_settings, $view_mode, $langcode);

      // Add only the site settings within the group to the cache tags.
      $cacheable_metadata = CacheableMetadata::createFromRenderArray($build);
      foreach ($site_settings as $site_setting) {
        $cacheable_metadata->addCacheableDependency($site_setting);
      }
      $cacheable_metadata->applyTo($build);
      return $build;
    }

    return [];
  }

  /**
   * Returns an array of site setting entity render arrays.
   */
  public function allSiteSettings(
    string $view_mode = 'full',
    ?string $langcode = NULL
  ): array {
    $site_settings = $this->pluginManager
      ->getActiveLoaderPlugin()
      ->loadAll(FALSE, $langcode);

    if (!empty($site_settings) && reset($site_settings) instanceof EntityInterface) {

      // Render the entities in the specified view mode and
      // language code.
      $build = $this->entityTypeManager
        ->getViewBuilder('site_setting_entity')
        ->viewMultiple($site_settings, $view_mode, $langcode);

      // Apply full list cache tag.
      CacheableMetadata::createFromRenderArray($build)
        ->addCacheTags(['site_setting_entity_list'])
        ->applyTo($build);
      return $build;
    }

    return [];
  }

  /**
   * Returns an array of site setting entity render arrays.
   */
  public function siteSettingsByGroup(
    string $group,
    string $view_mode = 'full',
    ?string $langcode = NULL
  ): array {
    $site_settings = $this->pluginManager
      ->getActiveLoaderPlugin()
      ->loadByGroup($group, $langcode);

    if (!empty($site_settings) && reset($site_settings) instanceof EntityInterface) {

      // Render the entities in the specified view mode and
      // language code.
      $build = $this->entityTypeManager
        ->getViewBuilder('site_setting_entity')
        ->viewMultiple($site_settings, $view_mode, $langcode);

      // Add only the site settings within the group to the cache tags.
      $cacheable_metadata = CacheableMetadata::createFromRenderArray($build);
      foreach ($site_settings as $site_setting) {
        $cacheable_metadata->addCacheableDependency($site_setting);
      }
      $cacheable_metadata->applyTo($build);
      return $build;
    }

    return [];
  }

}
